'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _regeneratorRuntime = _interopDefault(require('@babel/runtime/regenerator'));
var _asyncToGenerator = _interopDefault(require('@babel/runtime/helpers/asyncToGenerator'));
var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _possibleConstructorReturn = _interopDefault(require('@babel/runtime/helpers/possibleConstructorReturn'));
var _getPrototypeOf = _interopDefault(require('@babel/runtime/helpers/getPrototypeOf'));
var _inherits = _interopDefault(require('@babel/runtime/helpers/inherits'));
var _wrapNativeSuper = _interopDefault(require('@babel/runtime/helpers/wrapNativeSuper'));
var _objectSpread = _interopDefault(require('@babel/runtime/helpers/objectSpread'));
var deepEqual = _interopDefault(require('fast-deep-equal'));

function typeIsFunction(type) {
  return typeof type === 'function';
}

function getElementType(type) {
  return typeIsFunction(type) ? type.name : type;
}

function findStateNode(element) {
  if (element.stateNode instanceof HTMLElement) {
    return element.stateNode;
  }

  if (typeof element.type === 'function') {
    return element;
  }
}

function removeChildrenFromProps(props) {
  if (!props) {
    return props;
  }

  var returnProps = {};

  for (var key in props) {
    if (key !== 'children') {
      returnProps[key] = props[key];
    }
  }

  return returnProps;
}
function getElementState(elementState) {
  if (!elementState) {
    return {};
  }

  var baseState = elementState.baseState;

  if (baseState) {
    return baseState;
  }

  return elementState;
}
function buildNodeTree(element) {
  var tree = {
    children: []
  };

  var elementCopy = _objectSpread({}, element);

  if (!element) {
    return tree;
  }

  tree.name = getElementType(elementCopy.type);
  tree.node = findStateNode(elementCopy);
  tree.props = removeChildrenFromProps(elementCopy.memoizedProps);
  tree.state = getElementState(elementCopy.memoizedState);

  if (elementCopy.child) {
    tree.children.push(elementCopy.child);
    var child = elementCopy.child;

    while (child.sibling) {
      tree.children.push(child.sibling);
      child = child.sibling;
    }
  }

  tree.children = tree.children.map(function (child) {
    return buildNodeTree(child);
  });
  return tree;
}
function findInTree(tree, searchFn) {
  var selectFirst = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var returnArray = [];
  var stack = tree;

  while (stack.length || selectFirst && !returnArray.length) {
    var node = stack.shift();

    if (node.children && node.children.length) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = node.children[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var child = _step.value;

          if (searchFn(child)) {
            returnArray.push(child);
          }

          stack.push(child);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }

  return returnArray;
}
function findSelectorInTree(selectors, tree) {
  var selectFirst = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var searchFn = arguments.length > 3 ? arguments[3] : undefined;
  var treeArray = [tree];
  selectors.forEach(function (selector) {
    treeArray = findInTree(treeArray, function (child) {
      if (searchFn && typeof searchFn === 'function') {
        return searchFn(child);
      }

      return child.name === selector;
    }, selectFirst);
  });
  return treeArray;
}
function filterNodesBy(nodes, key, obj) {
  var filtered = [];

  var interator = function interator(el) {
    if (deepEqual(obj, el[key])) {
      filtered.push(el);
    }
  };

  nodes.forEach(interator);
  return filtered;
}

var RESQNodes =
/*#__PURE__*/
function (_Array) {
  _inherits(RESQNodes, _Array);

  function RESQNodes(nodes) {
    var _getPrototypeOf2;

    _classCallCheck(this, RESQNodes);

    return _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(RESQNodes)).call.apply(_getPrototypeOf2, [this].concat(_toConsumableArray(nodes))));
  }

  _createClass(RESQNodes, [{
    key: "byProps",
    value: function byProps(props) {
      var filtered = filterNodesBy(this, 'props', props);
      return new RESQNodes(filtered);
    }
  }, {
    key: "byState",
    value: function byState(state) {
      var filtered = filterNodesBy(this, 'state', state);
      return new RESQNodes(filtered);
    }
  }]);

  return RESQNodes;
}(_wrapNativeSuper(Array));

var RESQNode =
/*#__PURE__*/
function (_Object) {
  _inherits(RESQNode, _Object);

  function RESQNode(item, nodes) {
    var _this;

    _classCallCheck(this, RESQNode);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(RESQNode).call(this, item));
    _this._nodes = nodes;

    for (var key in item) {
      _this[key] = item[key];
    }

    return _this;
  }

  _createClass(RESQNode, [{
    key: "byProps",
    value: function byProps(props) {
      var filtered = filterNodesBy(this._nodes, 'props', props)[0];
      return new RESQNode(filtered, this._nodes);
    }
  }, {
    key: "byState",
    value: function byState(state) {
      var filtered = filterNodesBy(this._nodes, 'state', state)[0];
      return new RESQNode(filtered, this._nodes);
    }
  }]);

  return RESQNode;
}(_wrapNativeSuper(Object));

var RESQ =
/*#__PURE__*/
function () {
  function RESQ(selector, root) {
    _classCallCheck(this, RESQ);

    this.selectors = selector.split(' ').filter(function (el) {
      return !!el;
    }).map(function (el) {
      return el.trim();
    });
    this.rootComponent = root;
    this.tree = buildNodeTree(this.rootComponent);
  }

  _createClass(RESQ, [{
    key: "find",
    value: function find() {
      this.nodes = new RESQNodes(findSelectorInTree(this.selectors, this.tree, true));
      return new RESQNode(this.nodes[0], this.nodes);
    }
  }, {
    key: "findAll",
    value: function findAll() {
      return new RESQNodes(findSelectorInTree(this.selectors, this.tree));
    }
  }]);

  return RESQ;
}();

function waitToLoadReact(timeout, rootElSelector) {
  var findReactRoot = function findReactRoot() {
    var walker = document.createTreeWalker(document);

    if (rootElSelector) {
      return document.querySelector(rootElSelector);
    }

    while (walker.nextNode()) {
      if (walker.currentNode.hasOwnProperty('_reactRootContainer')) {
        return walker.currentNode;
      }
    }
  };

  return function () {
    return new Promise(function (resolve, reject) {
      var timedout = false;

      var tryToFindApp = function tryToFindApp() {
        var reactRoot = findReactRoot();

        if (reactRoot) {
          global.isReactLoaded = true;
          global.rootReactElement = reactRoot._reactRootContainer._internalRoot.current;
          return resolve();
        }

        if (timedout) {
          return;
        }

        setTimeout(tryToFindApp, 200);
      };

      tryToFindApp();
      setTimeout(function () {
        timedout = true;
        reject('Timed out');
      }, timeout);
    });
  }();
}

var resq$ =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee(selector) {
    var _ref2,
        timeout,
        rootElSelector,
        _args = arguments;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _ref2 = _args.length > 1 && _args[1] !== undefined ? _args[1] : {
              timeout: 5000
            }, timeout = _ref2.timeout, rootElSelector = _ref2.rootElSelector;

            if (global.isReactLoaded) {
              _context.next = 10;
              break;
            }

            _context.prev = 2;
            _context.next = 5;
            return waitToLoadReact(timeout, rootElSelector);

          case 5:
            _context.next = 10;
            break;

          case 7:
            _context.prev = 7;
            _context.t0 = _context["catch"](2);
            throw new Error('Could not find the root element of your application');

          case 10:
            return _context.abrupt("return", new RESQ(selector, global.rootReactElement).find());

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[2, 7]]);
  }));

  return function resq$(_x) {
    return _ref.apply(this, arguments);
  };
}();
var resq$$ =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee2(selector) {
    var _ref4,
        timeout,
        rootElSelector,
        _args2 = arguments;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _ref4 = _args2.length > 1 && _args2[1] !== undefined ? _args2[1] : {
              timeout: 5000
            }, timeout = _ref4.timeout, rootElSelector = _ref4.rootElSelector;

            if (global.isReactLoaded) {
              _context2.next = 10;
              break;
            }

            _context2.prev = 2;
            _context2.next = 5;
            return waitToLoadReact(timeout, rootElSelector);

          case 5:
            _context2.next = 10;
            break;

          case 7:
            _context2.prev = 7;
            _context2.t0 = _context2["catch"](2);
            throw new Error('Could not find the root element of your application');

          case 10:
            return _context2.abrupt("return", new RESQ(selector, global.rootReactElement).findAll());

          case 11:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[2, 7]]);
  }));

  return function resq$$(_x2) {
    return _ref3.apply(this, arguments);
  };
}();

exports.resq$ = resq$;
exports.resq$$ = resq$$;
